# coding: utf-8

from __future__ import absolute_import
from __future__ import unicode_literals

from django.core.exceptions import ImproperlyConfigured

from m3_auth.backends.base.remote import RemoteBackend
from m3_auth.backends.oauth2.exceptions import RefreshTokenError
from m3_auth.utils import check_dependency


if not all((
    check_dependency('requests', '2.23', '3.0'),
    check_dependency('oauthlib', '3.1', '4.0')
)):
    raise ImproperlyConfigured(
        'OAuthBackend depends on requests>=2.23,<3.0 and oauthlib>=3.1,<4.0. '
        'You need to check your environment.'
    )


class OAuthBackend(RemoteBackend):

    """
    Класс бэкенда для аутентификации по протоколу OAuth 2.0.
    Дополнительно реализует функционал обновления токена.
    """

    def refresh_token(self, request):
        """Метод обновляет токен. 
        Если возникает ошибка обновления токена, например если пользователь 
        отозвал разрешение, то выкидывает пользователя из системы.

        :param request: Запрос
        :type request: django.http.request.HttpRequest 

        :return: None
        """
        try:
            self.protocol.refresh_token(request)
        except RefreshTokenError:
            self._logout(request)
